<?php

namespace Drupal\content_entity_builder\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\content_entity_builder\ConfigurableBaseFieldConfigInterface;
use Drupal\content_entity_builder\ContentTypeInterface;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Provides a base form for base field.
 */
abstract class BaseFieldConfigFormBase extends FormBase {

  /**
   * The content entity type.
   *
   * @var \Drupal\content_entity_builder\ContentTypeInterface
   */
  protected $contentType;

  /**
   * The base field.
   *
   * @var \Drupal\content_entity_builder\BaseFieldConfigInterface
   */
  protected $baseField;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'base_field_form';
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\content_entity_builder\ContentTypeInterface $content_type
   *   The content_type.
   * @param string $base_field
   *   The base_field ID.
   *
   * @return array
   *   The form structure.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   */
  public function buildForm(array $form, FormStateInterface $form_state, ContentTypeInterface $content_type = NULL, $base_field = NULL) {
    $this->contentType = $content_type;
    try {
      $this->baseField = $this->prepareBaseField($base_field);
    }
    catch (PluginNotFoundException $e) {
      throw new NotFoundHttpException("Invalid base field id: '$base_field'.");
    }
    $request = $this->getRequest();

    if (!($this->baseField instanceof ConfigurableBaseFieldConfigInterface)) {
      throw new NotFoundHttpException();
    }

    $form['#attached']['library'][] = 'content_entity_builder/admin';
	/*
    $form['uuid'] = array(
      '#type' => 'hidden',
      '#value' => $this->tab->getUuid(),
    );
	*/
    $form['id'] = array(
      '#type' => 'hidden',
      '#value' => $this->baseField->getPluginId(),
    );
	
    $form['label'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#default_value' => $this->baseField->getLabel(),
    );
	
    $form['description'] = array(
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#default_value' => $this->baseField->getDescription(),
    );
	
    $form['required'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Required'),
      '#default_value' => $this->baseField->isRequired(),
    );
	/*
    $form['index'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Index'),
      '#default_value' => $this->baseField->hasIndex(),
    );	*/
    $form['default_value'] = [
      '#type' => 'details',
      '#title' => $this->t('Default value'),
	  '#open' => true,
    ];
	
    // Add handling for default value.
    if ($element = $this->baseField->buildDefaultValueForm(array(), $form_state)) {
      $element = array_merge($element, [
        '#type' => 'details',
        '#title' => $this->t('Default value'),
        '#open' => TRUE,
        '#tree' => TRUE,
        '#description' => $this->t('The default value for this field, used when creating new content.'),
      ]);

      $form['default_value'] = $element;
    }
/*	
    $default_value_form = $this->baseField->buildDefaultValueForm(array(), $form_state);
    if(!empty($default_value_form)){
		//$form['default_value'] = $default_value_element;
		 $form['default_value'] = $default_value_form;
	}	*/
    /*
    $form['default_value'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Default value'),
      '#default_value' => $this->baseField->getDefaultValue(),

    );	*/
	
    $form_state->setValue('has_data', $this->contentType->hasData());
    $form_state->setValue('applied', $this->baseField->isApplied());	
    $form['settings'] = $this->baseField->buildConfigurationForm(array(), $form_state);
    $form['settings']['#tree'] = TRUE;
	
    //$form['settings'] = [];
   //$subform_state = SubformState::createForSubform($form['settings'], $form, $form_state);
    //$form['settings'] = $this->baseField->buildConfigurationForm($form['settings'], $subform_state);
    //$form['settings']['#tree'] = TRUE;	
	//drupal_set_message('term_id:' . var_export($form['data']));
    // Check the URL for a weight, then the base field, otherwise use default.
    $form['weight'] = array(
      '#type' => 'hidden',
      '#value' => $request->query->has('weight') ? (int) $request->query->get('weight') : $this->baseField->getWeight(),
    );

    $form['actions'] = array('#type' => 'actions');
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#button_type' => 'primary',
    );
    $form['actions']['cancel'] = array(
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => $this->contentType->urlInfo('edit-form'),
      '#attributes' => ['class' => ['button']],
    );
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // The base field configuration is stored in the 'settings' key in the form,
    // pass that through for validation.
	$settings = $form_state->getValue('settings');
	if(!empty($settings)){
      //$base_field_settings = (new FormState())->setValues($form_state->getValue('settings'));
      $this->baseField->validateConfigurationForm($form['settings'], SubformState::createForSubform($form['settings'], $form, $form_state));
      // Update the original form values.
      //$form_state->setValue('settings', $base_field_settings->getValues());
	}
	
	$default_value = $form_state->getValue('default_value');
	if(!empty($default_value)){
      $base_field_default_value = (new FormState())->setValues($form_state->getValue('default_value'));
      //$this->baseField->validateConfigurationForm($form, $base_field_settings);
      // Update the original form values.
      //$form_state->setValue('default_value', $base_field_default_value->getValues());
	}	
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->cleanValues();

    // The base field configuration is stored in the 'settings' key in the form,
    // pass that through for submission.
	$settings = $form_state->getValue('settings');
	if(!empty($settings)){
      $base_field_settings = (new FormState())->setValues($settings);
      $this->baseField->submitConfigurationForm($form, $base_field_settings);
      // Update the original form values.
      $form_state->setValue('settings', $base_field_settings->getValues());
	
	  $this->baseField->setSettings($base_field_settings->getValues());
	}

	// $this->baseField->submitConfigurationForm($form['settings'], SubformState::createForSubform($form['settings'], $form, $form_state));
	$default_value = $form_state->getValue('default_value');
	if(!empty($default_value)){
      $base_field_default_value = (new FormState())->setValues($default_value);
      $this->baseField->submitDefaultValueForm($form, $base_field_default_value);
      // Update the original form values.
      //$form_state->setValue('settings', $base_field_settings->getValues());
	  // drupal_set_message(var_export($this->baseField->getDefaultValue(),true));
	  //$this->baseField->setDefaultValue( $this->baseField->getDefaultValue());
	}	
    //$this->baseField->setFieldName($form_state->getValue('field_name'));
    $this->baseField->setLabel($form_state->getValue('label'));

	$this->baseField->setRequired($form_state->getValue('required'));
	//drupal_set_message(var_export($form_state->getValue('index'), true));
	//$this->baseField->setIndex($form_state->getValue('index'));
	//$this->baseField->setDefaultValue($form_state->getValue('default_value'));
	//drupal_set_message(var_export($form_state->getValue('default_value')->getTimestamp(),true));
	//$this->baseField->setDefaultValue($form_state->getValue('default_value')->getTimestamp());
	$this->baseField->setDescription($form_state->getValue('description'));
    $this->contentType->save();

    //drupal_set_message($this->t('The base field was successfully applied.'));
    $form_state->setRedirectUrl($this->contentType->urlInfo('edit-form'));
  }

  /**
   * Converts a base_field ID into an object.
   *
   * @param string $base_field
   *   The base_field ID.
   *
   * @return \Drupal\content_entity_builder\BaseFieldConfigInterface
   *   The base_field object.
   */
  abstract protected function prepareBaseField($base_field);

}
